==> Cosmos7.60.tar.gz: Cosmos7.60/Tracking/cstack.f <==
c       stacking tracking information
c
      subroutine cpush(a)
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
#include  "Zevhnv.h"
      record /track/ a
      character*70  msg

c
c      call cifDead(a, icon)  ! this is moved to after cpop.
c                             ! so some ptcl may be stored even if 
c                             ! low energy.
c      if(icon .eq. 0) then
      if( a.p.code /=  0) then
         if( Stack_pos .ge. Max_stack_size) then
            write(msg,*) 'stack area full=',Max_stack_size
            call cerrorMsg(msg, 0)
         else
            Stack_pos = Stack_pos + 1
            Stack(Stack_pos) = a
         endif
      else
c           although very rare, 0 code appears;  neglect it (once /5 days)

         write(0,*) ' code=0  appeared subcode=',a.p.subcode
         write(0,*) ' charge=',a.p.charge
         write(0,*) ' px,py, pz, E, mass=',a.p.fm.p(1:4), a.p.mass
         write(0,*) ' neglected'
         write(0,*) 'ActiveMdl=',ActiveMdl
cc         call checkstat("in cpush")
      endif
      end
c------------------------------------------------------------
      subroutine cpop(a, remain)
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      record /track/ a

      integer remain
c
c         remain: int. is the number of ptcls remaining unprocessed
c                 including the current one to be processed now.
      if( Stack_pos .le. 0) then
         remain = 0
      else
         remain =  Stack_pos 
         a = Stack(Stack_pos)
         Stack_pos = Stack_pos -1
      endif
      end
      subroutine cgetStacked(stackpos, aTrack, icon)
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      integer stackpos  !  input.  stack pos from which track is
                        ! to be extracted.
      record /track/ aTrack  ! output extracted track.
      integer icon   ! output. 0 OK 1--> no. track at stackpos 
c          diff. from cpop;  Stack_pos is not affected.      
      if(stackpos .le. Stack_pos .and. stackpos .ge. 1) then
         aTrack = Stack(stackpos)
         icon = 0
      else
         icon = 1
      endif
      end

c---------------------------------------------------------
      subroutine cinitStack
c         initialize stack.
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      Stack_pos = 0
      end
      subroutine cgetCurrentStackPos(stackpos)
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      integer stackpos
      stackpos=Stack_pos
      end
      subroutine cresetStackPos(stackpos)
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      integer stackpos
      Stack_pos=stackpos
      end


      subroutine csortStack
c        sort stack dscending order 
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      real*8 erg(Max_stack_size)
      record /track/ sorted(Max_stack_size)
      integer  idx(Max_stack_size)
      
      integer i, j, k
      record /track/ temp
      do  i = 1,  Stack_pos
         erg(i) = Stack(i).p.fm.p(4)
      enddo
      
      call kqsortd(erg, idx, Stack_pos)
      call ksortinv(idx, Stack_pos)
      
      do i = 1, Stack_pos
         sorted(i) = Stack(idx(i))
      enddo

      do i = 1, Stack_pos
         Stack(i) = sorted(i)
      enddo
      end

==> Cosmos7.583.tar.gz: Cosmos7.583/Tracking/cstack.f <==
c       stacking tracking information
c
      subroutine cpush(a)
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
#include  "Zevhnv.h"
      record /track/ a
      character*70  msg

c
c      call cifDead(a, icon)  ! this is moved to after cpop.
c                             ! so some ptcl may be stored even if 
c                             ! low energy.
c      if(icon .eq. 0) then
      if( a.p.code /=  0) then
         if( Stack_pos .ge. Max_stack_size) then
            write(msg,*) 'stack area full=',Max_stack_size
            call cerrorMsg(msg, 0)
         else
            Stack_pos = Stack_pos + 1
            Stack(Stack_pos) = a
         endif
      else
c           although very rare, 0 code appears;  neglect it (once /5 days)

         write(0,*) ' code=0  appeared subcode=',a.p.subcode
         write(0,*) ' charge=',a.p.charge
         write(0,*) ' px,py, pz, E, mass=',a.p.fm.p(1:4), a.p.mass
         write(0,*) ' neglected for stacking in cstack.f'
         write(0,*) 'ActiveMdl=',ActiveMdl
cc         call checkstat("in cpush")
      endif
      end
c------------------------------------------------------------
      subroutine cpop(a, remain)
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      record /track/ a

      integer remain
c
c         remain: int. is the number of ptcls remaining unprocessed
c                 including the current one to be processed now.
      if( Stack_pos .le. 0) then
         remain = 0
      else
         remain =  Stack_pos 
         a = Stack(Stack_pos)
         Stack_pos = Stack_pos -1
      endif
      end
      subroutine cgetStacked(stackpos, aTrack, icon)
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      integer stackpos  !  input.  stack pos from which track is
                        ! to be extracted.
      record /track/ aTrack  ! output extracted track.
      integer icon   ! output. 0 OK 1--> no. track at stackpos 
c          diff. from cpop;  Stack_pos is not affected.      
      if(stackpos .le. Stack_pos .and. stackpos .ge. 1) then
         aTrack = Stack(stackpos)
         icon = 0
      else
         icon = 1
      endif
      end

c---------------------------------------------------------
      subroutine cinitStack
c         initialize stack.
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      Stack_pos = 0
      end
      subroutine cgetCurrentStackPos(stackpos)
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      integer stackpos
      stackpos=Stack_pos
      end
      subroutine cresetStackPos(stackpos)
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      integer stackpos
      Stack_pos=stackpos
      end


      subroutine csortStack
c        sort stack dscending order 
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      real*8 erg(Max_stack_size)
      record /track/ sorted(Max_stack_size)
      integer  idx(Max_stack_size)
      
      integer i, j, k
      record /track/ temp
      do  i = 1,  Stack_pos
         erg(i) = Stack(i).p.fm.p(4)
      enddo
      
      call kqsortd(erg, idx, Stack_pos)
      call ksortinv(idx, Stack_pos)
      
      do i = 1, Stack_pos
         sorted(i) = Stack(idx(i))
      enddo

      do i = 1, Stack_pos
         Stack(i) = sorted(i)
      enddo
      end
